#! /usr/bin/env python

# *** backprint.py ******************************************
# Auteur : Grard Swinnen, Verviers - Version du 04/01/2002
# Explications : voir http://www.ulg.ac.be/cifen/inforef/swi
# Copyright : licence GPL
# ***********************************************************

# Le prsent script est destin  tre automatiquement excut lorsqu'un
# utilisateur (un lve, en principe) se connecte au rseau local depuis
# son poste de travail Window$.
#
# Le nom de l'utilisateur, la machine utilise et l'instant de la connexion
# sont ajouts au fichier journal dont le nom et l'emplacement sont dfinis
# dans les variables logDir et logFileName (voir dbut du script :
# en principe, ce fichier devrait tre /var/log/samba/userlogons)
#
# La situation de l'utilisateur en matire de quotas d'espace disque
# (c..d. les quotas qui lui sont attribus et ceux qu'il a utiliss)
# est transcrite dans un petit fichier nomm <quotas.txt> , lequel est
# enregistr dans son rpertoire personnel.
#
# Accessoirement, le nom de l'utilisateur et les quotas d'espace disque qui
# le concernent sont "incrusts" dans l'image servant de fond d'cran
# ( condition que la librairie "Python Imaging Library" soit installe).
#
# Pour que tout ceci soit possible, il faut que le prsent script soit
# install dans le rpertoire /bin du serveur et soit rendu excutable.
# Il faut en outre qu'une ligne d'instruction telle que :
#      root preexec = /bin/backprint.py %U %m
# soit insre dans le fichier /etc/smb.conf, dans la description de l'un des
# partages "visits" automatiquement lors de la connexion, comme par exemple
# ceux qui correspondent  la dfinition de "lecteurs rseau" dans le
# "logon script" associ  l'utilisateur (le "logon script" est un fichier de
# type "groupe.bat" situ dans le rpertoire /home/netlogon du serveur.
#
# Si l'on souhaite l'incrustation des quotas dans le bureau de
# l'utilisateur, il faut en outre installer une police de caractres compatible
# avec PIL (fichiers timR24.pil & timR24.pbm ,  placer dans /bin)
# L'image utilise comme fond doit s'appeler "papierpeint" et tre installe
# dans le rpertoire <home> du groupe-classe.
# Note : la Python Imaging Library doit tre installe de prfrence dans
# le rpertoire /usr/lib/python (plutt que dans un sous-rpertoire PIL)
#
# Veuillez consulter les notes de G.Swinnen concernant les conventions
# adoptes pour regrouper les rpertoires personnels des lves par classes,
# ainsi que pour dfinir une image de fond de bureau.
#

import os, sys, string
from time import time, localtime
from os.path import exists

# Tentative d'importation de la Python Imaging Library
try:
    import Image, ImageDraw, ImageFont
except:
    PILp =0
else:
    PILp =1

cr = '\015\012'         # Codes de fin de ligne pour MSDOS/Windows
logDir ="/var/log/samba"
logFileName = "userlogons"

def descr(loguser):
   "description exacte de l'utilisateur suivant /etc/passwd"
   # <loguser> est l'identifiant entr par l'utilisateur au login.
   # Cette fonction restitue le nom UNIX correct (Samba accepte le login
   # sans tenir compte de la casse), le rpertoire personnel "technique"
   # et le commentaire dcrivant l'utilisateur.
   fp = open("/etc/passwd","r")
   realuser ='*inconnu*'
   comm =''
   homedir =''
   while 1:
      li = fp.readline()
      if len(li) == 0:           # dtection de la fin du fichier
         break
      # il faut comparer le nom d'utilisateur en ngligeant la casse :
      lp = string.split(li,":")
      if string.upper(lp[0]) == string.upper(loguser):
         # toilettage du nom complet (remplacement des souligns) :
         comm = string.replace(string.strip(lp[4]),"_"," ")
         homedir = lp[5]
         realuser = lp[0]
         break
   fp.close()
   return realuser, homedir, comm

def doit():
   "procdure principale"
   # Samba devrait avoir transmis le nom de login de l'utilisateur et le
   # nom du poste comme arguments, lors de l'appel du prsent script :
   user = sys.argv[1]
   machine = sys.argv[2]
   # Si le nom de machine contient un espace, concatner le 3e argument :
   if len(sys.argv) >3:
      machine = machine + sys.argv[3]

   # Description exacte de l'utilisateur suivant /etc/passwd :
   # (Particularit de Python : une fonction peut retourner un tuple)
   realuser, homedir, comm = descr(user)
   lt = localtime(time())           # tuple contenant date et heure

   # Mmorisation du login dans un fichier de traces :
   if not exists(logDir):
       os.mkdir(logDir, mode =0744)
   file = open(logDir + '/' + logFileName, "a")
   file.seek(0,2)           # positionnement  la fin du fichier
   file.write("%s %s %s (%s/%s/%s  %s:%s) : %s\n" % (machine,
              realuser,time(),lt[2],lt[1],lt[0],lt[3],lt[4],comm))
   file.close()

   # Gestion des quota :
   if homedir =='':
      sys.exit()           # pas de rpertoire personnel ??!!

   # on utilise un "pipe" pour lire les quotas et les placer dans une liste :
   lq = os.popen("quota " + realuser).readlines()

   if len(lq)<3:
      # pas de quotas pour cet utilisateur
      lv =[0,"non applicable",0,"pas de limite"]
   else:
      # seule la troisime ligne du listing nous intresse :
      q = lq[2]
      # on convertit cette chane en une liste de valeurs :
      lv = string.split(q)

   # Transcription des quotas dans le fichier destinataire (quotas.txt).

   # Suivant les conventions mises en place par G.Swinnen, le vrai
   # rpertoire personnel de l'utilisateur est un sous-rpertoire 
   # son nom dans le rpertoire personnel "technique" :
   realhomedir = homedir + "/" + realuser
   if exists(realhomedir):
      file = open(realhomedir + '/quotas.txt', "w")
      file.write("Espace disque rserv  l'utilisateur %s : %s"
                 % (realuser,cr))
      file.write("(nom complet = %s)%s%s" % (comm,cr,cr))
      file.write("Situation au %s/%s/%s :%s" % (lt[2],lt[1],lt[0],cr))
      file.write("Quotas allous (kb) : %s.%s" % (lv[3],cr))
      file.write("Quotas utiliss (kb) : %s.%s" % (lv[1],cr))
      file.close()

   # Incrustation d'un texte similaire directement dans le
   # papier peint du bureau :
   if PILp ==1:
      if exists(homedir +"/papierpeint"):
         im = Image.open(homedir +"/papierpeint")
      else:
         im = Image.new("P",(800,600), (150))
      # chargement d'une police compatible avec l'"imaging library" :
      fname ='/bin/timR24.pil'
      fnt = ImageFont.load(fname)
      # cration d'un objet pour dessiner dans l'image :
      draw = ImageDraw.ImageDraw(im)
      draw.setfont(fnt)
      # Nom de l'utilisateur + la date :
      text ="%s  %s/%s/%s" % (comm,lt[2],lt[1],lt[0])
      draw.text((60,30), text)
      text ="Quotas utiliss/allous (kb) : %s/%s." % (lv[1], lv[3])
      draw.text((60,70), text)
      im.save(homedir +"/background","BMP")
      del draw

if __name__ == "__main__":
   doit()
